/**
 * 当获取图片失败时会返回失败信息，是 text/plain 类型的数据
 * @param str - 图片内容
 * @returns
 */
export declare function isTextPlainBase64(str: string): boolean;
/**
 * 将网络图片转为 base64
 * @param url - 图片地址
 * @returns
 */
export declare function convertImageToBase64(url: string): Promise<string>;
/**
 * 使用 base64 的图片替换 img 标签的 src 或 image 标签的 href
 * @param node - html 节点或 svg 节点
 */
export declare function updateImageSrcOrHrefWithBase64Image(node: HTMLImageElement | SVGImageElement, attrName: 'src' | 'href'): Promise<void>;
/**
 * 使用 base64 的图片替换背景图片
 * @param node - html 节点
 * @param styleAttr - 样式属性名称
 */
export declare function updateBackgroundImageWithBase64Image(node: HTMLElement, url: string): Promise<void>;
/**
 * 更新图片数据
 * @param node - 节点
 */
export declare function updateImageSource(node: HTMLElement | SVGElement): Promise<void>;
/**
 * 重新复制canvas 用于在不裁剪原canvas的基础上通过拉伸方式达到自定义宽高目的
 * @param originCanvas HTMLCanvasElement
 * @param targetWidth number
 * @param targetHeight number
 */
export declare function copyCanvas(originCanvas: HTMLCanvasElement, targetWidth: number, targetHeight: number): HTMLCanvasElement;
